/*-
 * Copyright (c) 2004-2006
 *      Damien Bergamini <damien.bergamini@free.fr>. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice unmodified, this list of conditions, and the following
 *    disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $FreeBSD: src/sys/dev/ipw/if_ipwvar.h,v 1.4 2006/03/12 19:01:00 damien Exp $
 */

#define IPW_MAX_NSEG	1

struct ipw_soft_bd {
	struct ipw_bd	*bd;
	int		type;
#define IPW_SBD_TYPE_NOASSOC	0
#define IPW_SBD_TYPE_COMMAND	1
#define IPW_SBD_TYPE_HEADER	2
#define IPW_SBD_TYPE_DATA	3
	void		*priv;
};

struct ipw_soft_hdr {
	struct ipw_hdr			hdr;
	bus_dmamap_t			map;
	SLIST_ENTRY(ipw_soft_hdr)	next;
};

struct ipw_soft_buf {
	struct mbuf			*m;
	struct ieee80211_node		*ni;
	bus_dmamap_t			map;
	SLIST_ENTRY(ipw_soft_buf)	next;
};

struct ipw_rx_radiotap_header {
	struct ieee80211_radiotap_header wr_ihdr;
	uint8_t		wr_flags;
	uint16_t	wr_chan_freq;
	uint16_t	wr_chan_flags;
	uint8_t		wr_antsignal;
};

#define IPW_RX_RADIOTAP_PRESENT						\
	((1 << IEEE80211_RADIOTAP_FLAGS) |				\
	 (1 << IEEE80211_RADIOTAP_CHANNEL) |				\
	 (1 << IEEE80211_RADIOTAP_DBM_ANTSIGNAL))

struct ipw_tx_radiotap_header {
	struct ieee80211_radiotap_header wt_ihdr;
	uint8_t		wt_flags;
	uint16_t	wt_chan_freq;
	uint16_t	wt_chan_flags;
};

#define IPW_TX_RADIOTAP_PRESENT						\
	((1 << IEEE80211_RADIOTAP_FLAGS) |				\
	 (1 << IEEE80211_RADIOTAP_CHANNEL))

struct ipw_fw {
	struct firmware		*fp;		/* image handle */
	const char		*data;		/* firmware image data */
	size_t			size;		/* firmware image size */
};

struct ipw_softc {
	struct ifnet		*sc_ifp;
	struct ieee80211com	sc_ic;
	int			(*sc_newstate)(struct ieee80211com *,
				    enum ieee80211_state, int);
	device_t		sc_dev;

	struct mtx		sc_mtx;
	struct taskqueue	*sc_tq;		/* private task queue */
#if __FreeBSD_version < 700000
	struct proc		*sc_tqproc;
#endif
	struct task		sc_radiontask;	/* radio on processing */
	struct task		sc_radiofftask;	/* radio off processing */
	struct task		sc_scanstarttask;/* scan start processing */
	struct task		sc_scandonetask;/* scan completed processing */
	struct task		sc_assoclosttask;/* assoc lost processing */
	struct task		sc_downtask;	/* disassociate processing */
	struct task		sc_restarttask;	/* restart adapter processing */

	uint32_t		flags;
#define	IPW_FLAG_FW_LOADING	0x00000001	/* firmware being setup */
#define IPW_FLAG_FW_INITED	0x00000002	/* firmware initialized */
#define IPW_FLAG_SCANNING	0x00000004	/* busy scanning */
#define	IPW_FLAG_BUSY		0x00000008	/* busy sending a command */
#define	IPW_FLAG_ASSOCIATED	0x00000010	/* currently associated  */
#define	IPW_FLAG_ENABLED	0x00000020	/* adapter enabled */
#define	IPW_FLAG_LOCKED		0x00000040	/* adapter use locked */
#define IPW_FLAG_HAS_RFSWITCH	0x00010000	/* rfkill switch present */
#define	IPW_FLAG_HACK		0x00020000

	int			irq_rid;
	int			mem_rid;
	struct resource		*irq;
	struct resource		*mem;
	bus_space_tag_t		sc_st;
	bus_space_handle_t	sc_sh;
	void 			*sc_ih;

	enum ieee80211_opmode	fw_mode;	/* mode of current firmware */
	struct ipw_fw		fw_ucode;	/* cached microcode */
	struct ipw_fw		fw_fw;		/* cached firmware */

	bus_dma_tag_t		tbd_dmat;
	bus_dma_tag_t		rbd_dmat;
	bus_dma_tag_t		status_dmat;
	bus_dma_tag_t		cmd_dmat;
	bus_dma_tag_t		hdr_dmat;
	bus_dma_tag_t		txbuf_dmat;
	bus_dma_tag_t		rxbuf_dmat;

	bus_dmamap_t		tbd_map;
	bus_dmamap_t		rbd_map;
	bus_dmamap_t		status_map;
	bus_dmamap_t		cmd_map;

	bus_addr_t		tbd_phys;
	bus_addr_t		rbd_phys;
	bus_addr_t		status_phys;

	struct ipw_bd		*tbd_list;
	struct ipw_bd		*rbd_list;
	struct ipw_status	*status_list;

	struct ipw_cmd		cmd;
	struct ipw_soft_bd	stbd_list[IPW_NTBD];
	struct ipw_soft_buf	tx_sbuf_list[IPW_NDATA];
	struct ipw_soft_hdr	shdr_list[IPW_NDATA];
	struct ipw_soft_bd	srbd_list[IPW_NRBD];
	struct ipw_soft_buf	rx_sbuf_list[IPW_NRBD];

	SLIST_HEAD(, ipw_soft_hdr) free_shdr;
	SLIST_HEAD(, ipw_soft_buf) free_sbuf;

	uint32_t		table1_base;
	uint32_t		table2_base;

	uint32_t		txcur;
	uint32_t		txold;
	uint32_t		rxcur;
	int			txfree;

	int			curchan;	/* current h/w channel # */
	int			dwelltime;

	int			sc_tx_timer;
	int			sc_rfkill_timer;/* poll for rfkill change */
	int			sc_scan_timer;	/* scan request timeout */

	struct bpf_if		*sc_drvbpf;

	union {
		struct ipw_rx_radiotap_header th;
		uint8_t	pad[64];
	} sc_rxtapu;
#define sc_rxtap	sc_rxtapu.th
	int			sc_rxtap_len;

	union {
		struct ipw_tx_radiotap_header th;
		uint8_t	pad[64];
	} sc_txtapu;
#define sc_txtap	sc_txtapu.th
	int			sc_txtap_len;
};
